@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,6 @@
 
   (** Native audio converters *)
 
-let samplerate_converter () = Float_pcm.native_resample
+let samplerate_converter () = Audio.Mono.resample
 
 let () = Audio_converter.Samplerate.converters#register "native" samplerate_converter
