@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,13 +23,13 @@
 (** Values and types of the liquidsoap language. *)
 
 (** The type of a value. *)
-type kind = Lang_types.t
+type t = Lang_types.t
 
 (** {2 Values} *)
 
 (** A typed value. *)
-type value = { mutable t : kind ; value : in_value }
-and env = (string * value) list
+type value = { mutable t : t ; value : in_value }
+and full_env = (string * ((int*Lang_types.constraints) list * value)) list
 and in_value =
   | Unit
   | Bool    of bool
@@ -37,15 +37,18 @@ and in_value =
   | String  of string
   | Float   of float
   | Source  of Source.source
-  | Request of Request.raw Request.t option
-               (* Request.raw is arbitrary: this information is violated
-                * in the script language until it has enough expressivity. *)
+  | Request of Request.t
+  | Encoder of Encoder.format
   | List    of value list
   | Product of value * value
+  | Ref     of value ref
   | Fun     of (string * string * value option) list *
-               env * env * Lang_values.term
+               full_env * full_env * Lang_values.term
   | FFI     of (string * string * value option) list *
-               env * env * (env -> kind -> value)
+               full_env *
+               (full_env -> t -> value)
+
+type env = (string*value) list
 
 (** Get a string representation of a value. *)
 val print_value : value -> string
@@ -56,12 +59,12 @@ val iter_sources : (Source.source -> unit) -> value -> unit
 (** {2 Computation} *)
 
 (** Multiapply a value to arguments. *)
-val apply : value -> (string * value) list -> value
+val apply : value -> env -> t:t -> value
 
 (** {3 Helpers for source builtins} *)
 
 type proto =
-  (string * kind * value option * string option) list
+  (string * t * value option * string option) list
 
 (** Some flags that can be attached to operators. *)
 type doc_flag =
@@ -75,7 +78,7 @@ val add_builtin :
   descr:string ->
   ?flags:doc_flag list ->
   string ->
-  proto -> kind -> (env -> kind -> value) ->
+  proto -> t -> (env -> t -> value) ->
   unit
 
 (** Add an builtin to the language, more rudimentary version. *)
@@ -83,44 +86,76 @@ val add_builtin_base :
   category:string ->
   descr:string ->
   ?flags:doc_flag list ->
-  string -> in_value -> kind -> unit
+  string -> in_value -> t -> unit
 
 (** Category of an operator. *)
 type category =
   | Input (** Input. *)
   | Output (** Output. *)
+  | Conversions     (** Conversions of stream type *)
   | TrackProcessing (** Operations on tracks (e.g. mixing, etc.). *)
   | SoundProcessing (** Operations on sound (e.g. compression, etc.). *)
   | VideoProcessing (** Operations on video. *)
   | MIDIProcessing (** Operations on MIDI. *)
-  | Visualization (** Visializations of the sound. *)
+  | Visualization (** Visualizations of the sound. *)
   | SoundSynthesis (** Synthesis. *)
+  | Liquidsoap (** Liquidsoap. *)
+
+(** Get a string representation of a[ category]. *)
+val string_of_category : category -> string
 
 (** Get a string representation of a [doc_flag]. *)
 val string_of_flag : doc_flag -> string
 
+type lang_kind_format =
+  | Fixed of int | Variable of int | Any_fixed of int
+type lang_kind_formats =
+  | Unconstrained of t
+  | Constrained of
+      (lang_kind_format,lang_kind_format,lang_kind_format) Frame.fields
+
+val any_fixed : lang_kind_formats
+val any_fixed_with :
+        ?audio:int -> ?video:int -> ?midi:int -> unit -> lang_kind_formats
+
+val empty : lang_kind_formats
+val audio_any : lang_kind_formats
+val audio_mono : lang_kind_formats
+val audio_stereo : lang_kind_formats
+val audio_n : int -> lang_kind_formats
+val audio_variable : lang_kind_formats
+
+val video_only : lang_kind_formats
+val midi_n : int -> lang_kind_formats
+val midi_only : lang_kind_formats
+
+val audio_video_any : lang_kind_formats
+
+val kind_type_of_kind_format : fresh:int -> lang_kind_formats -> t
+
 (** Add an operator to the language and to the documentation. *)
 val add_operator :
   category:category ->
   descr:string ->
   ?flags:doc_flag list ->
+  ?active:bool ->
   string ->
-  proto -> (env -> kind -> Source.source) ->
+  proto ->
+  kind:lang_kind_formats ->
+  (env -> Frame.content_kind -> Source.source) ->
   unit
 
 (** {2 Manipulation of values} *)
 
+val to_unit : value -> unit
 val to_bool : value -> bool
 val to_string : value -> string
 val to_string_getter : value -> unit -> string
 val to_float : value -> float
 val to_float_getter : value -> unit -> float
 val to_source : value -> Source.source
-(** Expands a value representing a request
-  * Value here *must* be an audio request. 
-  * Assert false if not.. *)
-val to_request : value -> Request.audio Request.t option
-val to_request_raw : value -> Request.raw Request.t option
+val to_format : value -> Encoder.format
+val to_request : value -> Request.t
 val to_int : value -> int
 val to_list : value -> value list
 val to_product : value -> value * value
@@ -130,53 +165,78 @@ val to_int_list : value -> int list
 val to_source_list : value -> Source.source list
 
 (** [assoc x n l] returns the [n]-th [y] such that [(x,y)] is in the list [l].
-  * This is useful for retreiving arguments of a function. *)
+  * This is useful for retrieving arguments of a function. *)
 val assoc : 'a -> int -> ('a * 'b) list -> 'b
 
-val int_t      : kind
-val unit_t     : kind
-val float_t    : kind
-val bool_t     : kind
-val string_t   : kind
-val source_t   : kind
-val request_t  : kind
-val list_t     : kind -> kind
-val product_t  : kind -> kind -> kind
+val int_t      : t
+val unit_t     : t
+val float_t    : t
+val bool_t     : t
+val string_t   : t
+val product_t  : t -> t -> t
+val of_product_t : t -> t * t
+
+val list_t     : t -> t
+val of_list_t  : t -> t
+
+val zero_t     : t
+val variable_t : t
+val succ_t     : t -> t
+val type_of_int : int -> t
+
+val request_t    : t -> t
+val of_request_t : t -> t
+
+val source_t    : t -> t
+val of_source_t : t -> t
+
+val format_t : t -> t
+
+val frame_kind_t : audio:t -> video:t -> midi:t -> t
+val of_frame_kind_t : t -> (t,t,t) Frame.fields
+
+val frame_kind_of_kind_type : t -> Frame.content_kind
+
+val kind_type_of_frame_kind : Frame.content_kind -> t
 
 (** [fun_t args r] is the type of a function taking [args] as parameters
   * and returning values of type [r].
   * The elements of [r] are of the form [(b,l,t)] where [b] indicates if
   * the argument is optional, [l] is the label of the argument ([""] means no
   * label) and [t] is the type of the argument. *)
-val fun_t      : (bool * string * kind) list -> kind -> kind
-val univ_t     : ?constraints:Lang_types.constraints -> int -> kind
+val fun_t      : (bool * string * t) list -> t -> t
+val univ_t     : ?constraints:Lang_types.constraints -> int -> t
 
 (** A shortcut for lists of pairs of strings. *)
-val metadata_t : kind
+val metadata_t : t
 
 (** A string getter. The argument is the number of the universal type parameter
   * (should be >= 1). *)
-val string_getter_t : int -> kind
+val string_getter_t : int -> t
 (** A float getter. The argument is the number of the universal type parameter
   * (should be >= 1). *)
-val float_getter_t : int -> kind
+val float_getter_t : int -> t
 
 val unit : value
 val int : int -> value
 val bool : bool -> value
 val float : float -> value
 val string : string -> value
-val list : value list -> value
+val list : t:t -> value list -> value
 val source : Source.source -> value
-val request : Request.raw Request.t option -> value
+val request : Request.t -> value
 val product : value -> value -> value
-val val_fun :
-      (string * string * value option) list -> (env -> kind -> value) -> value
 
-(** Specialized builder for constant functions.
+(** Build a function from an OCaml function.
+  * Items in the prototype indicate the label, type and optional
+  * values. *)
+val val_fun : (string * string * t * value option) list -> ret_t:t ->
+              (env -> t -> value) -> value
+
+(** Build a constant function.
   * It is slightly less opaque and allows the printing of the closure
   * when the constant is ground. *)
-val val_cst_fun : (string * string * value option) list -> value -> value
+val val_cst_fun : (string * t * value option) list -> value -> value
 
 (** Convert a metadata packet to a list associating strings to strings. *)
 val metadata : Frame.metadata -> value
@@ -185,16 +245,27 @@ val metadata : Frame.metadata -> value
 
 exception Invalid_value of value * string
 
+(** More informative version of clocks exceptions from Source,
+  * used for re-raising and displaying better error messages *)
+exception Clock_conflict of (Lang_types.pos option * string * string)
+exception Clock_loop of (Lang_types.pos option * string * string)
+
 (** {2 Main script evaluation} *)
 
 (** Load the external libraries. *)
 val load_libs       : ?parse_only:bool -> unit -> unit
 
 (** Evaluate a script from an [in_channel]. *)
-val from_in_channel : ?parse_only:bool -> in_channel -> unit
+val from_in_channel : ?parse_only:bool -> lib:bool -> in_channel -> unit
 
 (** Evaluate a script from a file. *)
-val from_file       : ?parse_only:bool -> string -> unit
+val from_file       : ?parse_only:bool -> lib:bool -> string -> unit
 
 (** Evaluate a script from a string. *)
-val from_string     : ?parse_only:bool -> string -> unit
+val from_string     : ?parse_only:bool -> lib:bool -> string -> unit
+
+(** Interactive loop: read from command line, eval, print and loop. *)
+val interactive : unit -> unit
+
+(** Evaluate a string *)
+val eval : string -> value option
