@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
  *****************************************************************************)
 
-open Root
 open Dtools
 open Printf
 
@@ -34,9 +33,18 @@ let () =
   Configure.conf#plug "init" Init.conf ;
   Configure.conf#plug "log" Log.conf
 
-(* Should we not_run the outputs? *)
+(* Should we not run the active sources? *)
 let dont_run = ref false
 
+(* Should we start even without active sources? *)
+let force_start =
+  Dtools.Conf.bool
+    ~p:(Dtools.Init.conf#plug "force_start") ~d:false
+    "Start liquidsoap even without any active source"
+    ~comments:[
+      "This should be reserved for advanced dynamic uses of liquidsoap."
+    ]
+
 (* Do not run, don't even check the scripts. *)
 let parse_only = ref false
 
@@ -46,20 +54,69 @@ let pervasives = ref true
 (* Have we been used for an other purpose than streaming? *)
 let secondary_task = ref false
 
-(** [check_pervasives] should be called before loading a script or looking up
+(* Shall we start an interactive interpreter (REPL) *)
+let interactive = ref false
+
+(** [load_libs] should be called before loading a script or looking up
   * the documentation, to make sure that pervasive libraries have been loaded,
   * unless the user explicitly opposed to it. *)
-let check_pervasives =
+let load_libs =
   let loaded = ref false in
     fun () ->
       if !pervasives && not !loaded then begin
-        Lang.load_libs ~parse_only:!parse_only () ;
-        loaded := true
+        let save = !Configure.display_types in
+          Configure.display_types := false ;
+          Lang.load_libs ~parse_only:!parse_only () ;
+          loaded := true ;
+          Configure.display_types := save
       end
 
+(** Evaluate a script or expression.
+  * This used to be done immediately, which made it possible to
+  * write things like "liquidsoap myscript.liq -h myop" and get
+  * some doc on an operator.
+  * Now, we delay each evaluation because the last item has to be treated
+  * as a non-library, ie. all defined variables should be used.
+  * By default the last item is (the only one) not treated as a library,
+  * for better diagnosis, but this can be disabled (which is useful
+  * when --checking a lib). *)
+
+let last_item_lib = ref false
+
+let do_eval, eval =
+  let delayed = ref None in
+  let eval src ~lib =
+    load_libs () ;
+    match src with
+      | `StdIn ->
+          Log.conf_stdout#set_d (Some true) ;
+          Log.conf_file#set_d (Some false) ;
+          Lang.from_in_channel ~lib ~parse_only:!parse_only stdin
+      | `Expr_or_File expr when (not (Sys.file_exists expr)) ->
+          Log.conf_stdout#set_d (Some true) ;
+          Log.conf_file#set_d (Some false) ;
+          Lang.from_string ~lib ~parse_only:!parse_only expr
+      | `Expr_or_File f ->
+          let basename = Filename.basename f in
+          let basename =
+            try Filename.chop_extension basename with _ -> basename
+          in
+            Configure.var_script := basename ;
+            Lang.from_file ~lib ~parse_only:!parse_only f
+  in
+  let force ~lib =
+    match !delayed with Some f -> f ~lib ; delayed := None | None -> ()
+  in
+    force,
+    (fun src -> force ~lib:true ; delayed := Some (eval src))
+
+let load_libs () =
+  do_eval ~lib:true ;
+  load_libs ()
+
 let plugin_doc name =
   secondary_task := true ;
-  check_pervasives () ;
+  load_libs () ;
   let found = ref false in
     List.iter
       (fun (lbl, i) ->
@@ -81,28 +138,15 @@ let plugin_doc name =
     if not !found then
       Printf.printf "Plugin not found!\n%!"
 
-let eval src =
-  check_pervasives () ;
-  match src with
-    | `StdIn ->
-        Log.conf_stdout#set_d (Some true) ;
-        Log.conf_file#set_d (Some false) ;
-        Lang.from_in_channel ~parse_only:!parse_only stdin
-    | `Expr_or_File expr when (not (Sys.file_exists expr)) ->
-        Log.conf_stdout#set_d (Some true) ;
-        Log.conf_file#set_d (Some false) ;
-        Lang.from_string ~parse_only:!parse_only expr
-    | `Expr_or_File f ->
-        let basename = Filename.basename f in
-        let basename =
-          try Filename.chop_extension basename with _ -> basename
-        in
-          Configure.var_script := basename ;
-          Lang.from_file ~parse_only:!parse_only f
-
 let process_request s =
+  load_libs () ;
   secondary_task := true ;
-  let req = Utils.get_some (Request.create s) in
+  let kind =
+    { Frame.audio = Frame.Variable ;
+      Frame.video = Frame.Variable ;
+      Frame.midi = Frame.Variable }
+  in
+  let req = Request.create ~kind s in
     match Request.resolve req 20. with
       | Request.Failed ->
           Printf.printf "Request resolution failed.\n" ;
@@ -185,35 +229,52 @@ struct
     in
       aux (string_of_path prefix) (t#path prefix)
 
-  let descr ?(prefix=[]) t =
-    let rec aux prefix t =
+  let descr ?(liqi=false) ?(prefix=[]) t =
+    let rec aux level prefix t =
       let p s = if prefix = "" then s else prefix ^ "." ^ s in
       let subs =
-        List.map (function s -> aux (p s) (t#path [s])) t#subs
+        List.map (function s -> aux (level+1) (p s) (t#path [s])) t#subs
+      in
+      let title,default,set,comment = 
+        if liqi then
+          Printf.sprintf "h%d. %s\n",
+          Printf.sprintf "Default: @%s@\n",
+          Printf.sprintf "%%%%\n\
+                          set(%S,%s)\n\
+                          %%%%\n",
+          (fun l -> String.concat ""
+                     (List.map (fun s -> Printf.sprintf "%s\n" s) l)) 
+        else
+          (fun _ ->
+             if t#kind = None then
+               Printf.sprintf "### %s\n\n"
+             else
+               Printf.sprintf "## %s\n"),
+          Printf.sprintf "# Default: %s\n",
+          Printf.sprintf "set(%S,%s)\n",
+          (fun l -> Printf.sprintf "# Comments:\n%s"
+                  (String.concat ""
+                    (List.map (fun s -> Printf.sprintf "#  %s\n" s) l)))
       in
         begin match t#kind, get_string t with
-        | None, None -> Printf.sprintf "### %s\n\n" t#descr
+        | None, None -> title level t#descr
         | Some _, Some p ->
-            Printf.sprintf "## %s\n"
-            t#descr ^
+            title level t#descr ^
             begin match get_d_string t with
               | None -> ""
-              | Some d -> Printf.sprintf "# Default: %s\n" d
+              | Some d -> default d
             end ^
-            Printf.sprintf "set(%S,%s)\n" prefix p ^
+            set prefix p ^
             begin match t#comments with
               | [] -> ""
-              | l ->
-                  "# Comments:\n" ^
-                  String.concat ""
-                    (List.map (fun s -> Printf.sprintf "#  %s\n" s) l)
+              | l -> comment l
             end ^
             "\n"
         | _ -> ""
         end ^
         String.concat "" subs
     in
-      aux (string_of_path prefix) (t#path prefix)
+      aux 2 (string_of_path prefix) (t#path prefix)
 
   let descr_key t p =
     try
@@ -221,7 +282,9 @@ struct
       exit 0
     with
       | Dtools.Conf.Unbound _ ->
-          Printf.eprintf "The key '%s' is not a valid configuration key.\n%!" p;
+          Printf.eprintf
+            "The key '%s' is not a valid configuration key.\n%!"
+            p ;
           exit 1
 
   let args t =
@@ -233,6 +296,10 @@ struct
       Arg.Unit (fun () ->
         print_string (descr t); exit 0),
       "Display a described table of the configuration keys.";
+      ["--conf-descr-liqi"],
+      Arg.Unit (fun () ->
+        print_string (descr ~liqi:true t); exit 0),
+      "Display a described table of the configuration keys in liqi (documentation wiki) format.";
       ["--conf-dump"],
       Arg.Unit (fun () ->
         print_string (dump t); exit 0),
@@ -264,17 +331,19 @@ let format_doc s =
   in
     String.concat "\n" s
 
+let log = Log.make ["main"]
+
 let options =
   List.fold_left
-    ( fun l (la,b,c) ->
+    (fun l (la,b,c) ->
         let ta = List.hd (List.rev la) in
-        let expand = List.map
-                       (fun a -> (a,b,(if a = ta then
-                                         "\n" ^ format_doc c
-                                       else
-                                         "")))
-                       la in
-          l@expand ) []
+        let expand =
+          List.map
+            (fun a -> (a,b,
+                       if a = ta then "\n" ^ format_doc c else ""))
+            la
+        in
+          l@expand) []
     (let opts = [
       ["-"],
       Arg.Unit (fun () -> eval `StdIn),
@@ -294,6 +363,14 @@ let options =
                   dont_run := true),
       "Check and evaluate scripts but do not perform any streaming." ;
 
+      ["-cl";"--check-lib"],
+      Arg.Unit (fun () ->
+                  last_item_lib := true ;
+                  secondary_task := true ;
+                  dont_run := true),
+      "Like --check but treats all scripts and expressions as libraries, \
+       so that unused toplevel variables are not reported." ;
+
       ["-p";"--parse-only"],
       Arg.Unit (fun () ->
                   secondary_task := true ;
@@ -308,15 +385,37 @@ let options =
       Arg.Unit (fun () -> Log.conf_stdout#set true),
       "Print log messages on standard output." ;
 
+      ["-f";"--force-start"],
+      Arg.Unit (fun () -> force_start#set true),
+      "For advanced dynamic uses: force liquidsoap to start \
+       even when no active source is initially defined." ;
+
       ["--debug"],
       Arg.Unit (fun () -> Log.conf_level#set (max 4 Log.conf_level#get)),
-      "Print debugging log messages." ;
-
-      ["-d";"--daemon"],
-      Arg.Unit (fun f -> Init.conf_daemon#set true),
-      "Run in daemon mode." ;
-
-      ["-t";"--enable-telnet"],
+      "Print debugging log messages." ]
+      @
+      (if Configure.dynlink then
+          [["--dynamic-plugins-dir"],
+              Arg.String (fun d ->
+              Dyntools.load_plugins_dir d),
+           "Directory where to look for plugins."]
+        else
+          [])
+      @ 
+      [["--errors-as-warnings"],
+      Arg.Set Lang_values.errors_as_warnings,
+      "Issue warnings instead of fatal errors for unused variables \
+       and ignored expressions. If you are not sure about it, it is better \
+       to not use it." ]
+      @
+      (* Unix.fork is not implemented in Win32. *) 
+      (if Sys.os_type <> "Win32" then
+        [["-d";"--daemon"],
+         Arg.Unit (fun f -> Init.conf_daemon#set true),
+         "Run in daemon mode."]
+       else [])
+      @
+      [["-t";"--enable-telnet"],
       Arg.Unit (fun _ -> Server.conf_telnet#set true),
       "Enable the telnet server." ;
 
@@ -335,7 +434,7 @@ let options =
       ["--list-plugins-xml"],
       Arg.Unit (fun () ->
                   secondary_task := true ;
-                  check_pervasives () ;
+                  load_libs () ;
                   Doc.print_xml (Plug.plugs:Doc.item)),
       Printf.sprintf
         "List all plugins (builtin scripting values, \
@@ -345,7 +444,7 @@ let options =
       ["--list-plugins"],
       Arg.Unit (fun () ->
                   secondary_task := true ;
-                  check_pervasives () ;
+                  load_libs () ;
                   Doc.print (Plug.plugs:Doc.item)),
       Printf.sprintf
         "List all plugins (builtin scripting values, \
@@ -364,25 +463,55 @@ let options =
       ["--version"],
       Arg.Unit (fun () ->
                   Printf.printf
-                    "Liquidsoap %s%s.\n\
-                     Copyright (c) 2003-2009 Savonet team\n\
+                    "Liquidsoap %s%s\n\
+                     Copyright (c) 2003-2011 Savonet team\n\
                      Liquidsoap is open-source software, \
                      released under GNU General Public License.\n\
                      See <http://savonet.sf.net> for more information.\n"
                     Configure.version SVN.rev ;
                   exit 0),
-     "Display liquidsoap's version." ;
+      "Display liquidsoap's version." ;
 
-    ["--"],
-    Arg.Unit (fun () -> Arg.current := Array.length Shebang.argv - 1),
-    "Stop parsing the command-line and pass subsequent items to the script." ]
+      ["--interactive"],
+      Arg.Set interactive,
+      "Start an interactive interpreter." ;
 
-     in opts@LiqConf.args Configure.conf@Configure.dynliq_option)
+      ["--"],
+      Arg.Unit (fun () -> Arg.current := Array.length Shebang.argv - 1),
+      "Stop parsing the command-line and pass subsequent items to the script."
 
-let log = Log.make ["main"]
+      ]
 
-let () =
-  log#f 3 "Liquidsoap %s%s" Configure.version SVN.rev
+     in 
+     let opts = 
+       List.sort (fun (x,_,_) (y,_,_) -> compare x y) opts
+     in opts@LiqConf.args Configure.conf)
+
+let () = 
+  log#f 3 "Liquidsoap %s%s" Configure.version SVN.rev ;
+  log#f 3 "Using:%s" Configure.libs_versions ;
+  if Configure.svn_snapshot then
+    List.iter (log#f 2 "%s")
+      ["";
+       "DISCLAIMER: This version of Liquidsoap has been";
+       "compiled from a snapshot of the development code.";
+       "As such, it should not be used in production";
+       "unless you know what you are doing!";
+       "";
+       "We are, however, very interested in any feedback";
+       "about our development code and committed to fix";
+       "issues as soon as possible.";
+       "";
+       "If you are interested in collaborating to";
+       "the development of Liquidsoap, feel free to";
+       "drop us a mail at <savonet-devl@lists.sf.net>";
+       "or to join the #savonet IRC channel on Freenode.";
+       "";
+       "Please send any bug report or feature request";
+       "at <http://dev.sourcefabric.org/browse/LS>.";
+       "";
+       "We hope you enjoy this snapshot build of Liquidsoap!";
+       ""]
 
 (** Just like Arg.parse_argv but with Arg.parse's behavior on errors.. *)
 let parse argv l f msg =
@@ -398,6 +527,13 @@ let absolute s =
   else
     s
 
+(* Load plugins and dynamic loaded libraries: 
+ * these should be loaded as early as possible since we want to be
+ * able to use them in scripts... *)
+let () =
+  Configure.load_dynlinks ();
+  Configure.load_plugins_dir Configure.plugins_dir
+
 (* Startup *)
 let () =
 
@@ -408,62 +544,92 @@ let () =
   Init.conf_daemon_pidfile#set_d (Some true) ;
   Init.conf_daemon_pidfile_path#set_d (Some "<sysrundir>/<script>.pid") ;
 
+  (* We only allow evaluation of
+   * lazy configuration keys now. *)
+  Frame.allow_lazy_config_eval ();
+
   (* Parse command-line, and notably load scripts. *)
   parse Shebang.argv options (fun s -> eval (`Expr_or_File s)) usage ;
+  do_eval ~lib:!last_item_lib
+
+(* When the log/pid paths have their definitive values,
+ * expand substitutions and check directories.
+ * This should be ran just before Dtools init. *)
+let check_directories () =
 
   (* Now that the paths have their definitive value, expand <shortcuts>. *)
   let subst conf = conf#set (Configure.subst_vars conf#get) in
-    subst Log.conf_file_path ;
-    subst Init.conf_daemon_pidfile_path
+  subst Log.conf_file_path ;
+  subst Init.conf_daemon_pidfile_path ;
 
-(* Check that directories used by Dtools exist. *)
-let check_directories () =
   let check_dir conf_path kind =
     let path = conf_path#get in
     let dir = Filename.dirname path in
-      if not (Sys.file_exists dir) then begin
-        Printf.printf "FATAL ERROR: %s directory %S does not exist.\n"
-          kind dir ;
+      if not (Sys.file_exists dir) then
         let routes = Configure.conf#routes conf_path#ut in
           Printf.printf
-            "To change it, add the following to your script:\n";
-          Printf.printf "  set(%S, \"<path>\")\n"
-            (Conf.string_of_path (List.hd routes)) ;
+            "FATAL ERROR: %s directory %S does not exist.\n\
+             To change it, add the following to your script:\n\
+            \  set(%S, \"<path>\")\n"
+            kind dir (Conf.string_of_path (List.hd routes)) ;
           exit 1
-      end
   in
     if Log.conf_file#get then
       check_dir Log.conf_file_path "Log" ;
     if Init.conf_daemon#get && Init.conf_daemon_pidfile#get then
-      check_dir Init.conf_daemon_pidfile_path "Pid"
+      check_dir Init.conf_daemon_pidfile_path "PID"
 
 (* Now that outputs have been defined, we can start the main loop. *)
 let () =
-  let root = ref (Thread.self ()) in
-    let cleanup () =
-      log#f 3 "Shutdown started!" ;
-      Root.shutdown := true ;
-      (* It's the root's job to shutdown sources,
-       * but we might have to replace it if it died. *)
-      if not (Tutils.running "root" !root) then Root.sleep () ;
-      log#f 3 "Waiting for threads to terminate..." ;
-      Tutils.join_all () ;
-      log#f 3 "Cleaning downloaded files..." ;
-      Request.clean ()
-    in
-    let main () =
-      root := Tutils.create Root.start () "root" ;
-      Tutils.main ()
-    in
-      ignore (Init.at_stop cleanup) ;
-      if Source.has_outputs () then
-        if not !dont_run then begin
-          check_directories () ;
-          Init.init ~prohibit_root:true main
-        end else
-          cleanup ()
-      else
-        (* If there's no output and no secondary task has been performed,
-         * warn the user that his scripts didn't define any output. *)
-        if not !secondary_task then
-          Printf.printf "No output defined, nothing to do.\n"
+  let cleanup () =
+    log#f 3 "Shutdown started!" ;
+    Clock.stop () ;
+    log#f 3 "Waiting for threads to terminate..." ;
+    Tutils.join_all () ;
+    log#f 3 "Cleaning downloaded files..." ;
+    Request.clean ()
+  in
+  let main () =
+    (* See http://caml.inria.fr/mantis/print_bug_page.php?bug_id=4640
+     * for this: we want Unix EPIPE error and not SIGPIPE, which
+     * crashes the program.. *)
+    if Sys.os_type <> "Win32" then
+     begin
+      Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+      ignore (Unix.sigprocmask Unix.SIG_BLOCK [Sys.sigpipe])
+     end;
+    (* On Windows we need to initiate shutdown ourselves by catching INT
+     * since dtools doesn't do it. *)
+    if Sys.os_type = "Win32" then
+      Sys.set_signal Sys.sigint
+        (Sys.Signal_handle (fun _ -> Tutils.shutdown ())) ;
+    (* TODO if start fails (e.g. invalid password or mountpoint) it
+     *   raises an exception and dtools catches it so we don't get
+     *   a backtrace (by default at least). *)
+    Clock.start () ;
+    Tutils.main ()
+  in
+    ignore (Init.at_stop cleanup) ;
+    if !interactive then begin
+      Log.conf_stdout#set_d (Some false) ;
+      Log.conf_file#set_d (Some true) ;
+      let default_log =
+        Filename.temp_file
+          (Printf.sprintf "liquidsoap-%d-" (Unix.getpid ())) ".log"
+      in
+      Log.conf_file_path#set_d (Some default_log) ;
+      ignore (Init.at_stop (fun _ -> Sys.remove default_log)) ;
+      check_directories () ;
+      ignore (Thread.create Lang.interactive ()) ;
+      Init.init main
+    end else if Source.has_outputs () || force_start#get then
+      if not !dont_run then begin
+        check_directories () ;
+        Init.init ~prohibit_root:true main
+      end else
+        cleanup ()
+    else
+      (* If there's no output and no secondary task has been performed,
+       * warn the user that his scripts didn't define any output. *)
+      if not !secondary_task then
+        Printf.printf "No output defined, nothing to do.\n"
