@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,15 +25,15 @@
 
 open Source
 
-class delay (source:source) delay =
+class delay ~kind ~initial (source:source) delay =
 object (self)
-  inherit operator [source] as super
+  inherit operator kind [source] as super
 
   method stype = Fallible
   method remaining = source#remaining
   method abort_track = source#abort_track
 
-  val mutable last = 0.
+  val mutable last = if initial then Unix.time () else 0.
   val mutable in_track = false
 
   method private delay_ok = (Unix.time ()) -. last >= delay
@@ -50,18 +50,23 @@ object (self)
 
 end
 
-let () = 
+let () =
+  let kind = Lang.univ_t 1 in
   Lang.add_operator "delay"
-    [ "", Lang.float_t, None,
-      (Some ("The source won't be ready less than this amount of seconds "^
-             "after any end of track")) ;
-      "", Lang.source_t, None, None
+    [ "initial", Lang.bool_t, Some (Lang.bool false),
+      Some "Start in unavailable state, as if a track had just finished." ;
+      "", Lang.float_t, None,
+      Some "The source won't be ready less than this amount of seconds \
+            after any end of track" ;
+      "", Lang.source_t kind, None, None
     ]
     ~category:Lang.TrackProcessing
-    ~descr:("Prevents the child from being ready again too fast after "^
-            "a end of track")
-    (fun p _ ->
+    ~descr:"Prevents the child from being ready again too fast after \
+            a end of track"
+    ~kind:(Lang.Unconstrained kind)
+    (fun p kind ->
        let f n = Lang.assoc "" n p in
        let d = Lang.to_float (f 1) in
        let s = Lang.to_source (f 2) in
-         new delay s d)
+       let initial = Lang.to_bool (List.assoc "initial" p) in
+         new delay ~kind ~initial s d)
