@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ open Source
 (** In this file [length]s are in samples, [threshold] are RMS (in [0.;1.]). *)
 
 
-class virtual base ~length ~threshold = 
+class virtual base ~length ~threshold =
 object(self)
 
   val mutable blank_len = 0
@@ -35,9 +35,9 @@ object(self)
 
   method virtual private on_noise : unit
 
-  method private check_blank s p0 = 
+  method private check_blank s p0 =
     let len = AFrame.position s - p0 in
-    let rms = Float_pcm.rms (AFrame.get_float_pcm s) p0 len in
+    let rms = AFrame.rms s p0 len in
     let noise = ref false in
       Array.iter (fun r -> if r > threshold then noise := true) rms;
       if !noise then
@@ -53,17 +53,18 @@ object(self)
 
 end
 
-class on_blank ~length ~threshold ~on_blank ~on_noise source =
+class on_blank ~kind ~length ~threshold
+  ~track_sensitive ~on_blank ~on_noise source =
 object (self)
-  inherit operator [source]
+  inherit operator kind [source]
   inherit base ~length ~threshold as base
 
   val mutable in_blank = true
 
   method stype = source#stype
   method is_ready = source#is_ready
-  method abort_track = 
-    source#abort_track ; 
+  method abort_track =
+    source#abort_track ;
     base#abort_track
   method remaining = source#remaining
 
@@ -72,32 +73,34 @@ object (self)
   method private get_frame ab =
     let p0 = AFrame.position ab in
       source#get ab ;
-      if AFrame.is_partial ab || p0 > 0 then blank_len <- 0 else begin
+      if track_sensitive && (AFrame.is_partial ab || p0 > 0) then
+        blank_len <- 0
+      else begin
         self#check_blank ab p0 ;
         if blank_len <= length then begin
           if in_blank then begin
-            ignore (Lang.apply on_noise []) ;
+            ignore (Lang.apply ~t:Lang.unit_t on_noise []) ;
             in_blank <- false
           end
         end else
           if not in_blank then begin
             in_blank <- true ;
-            ignore (Lang.apply on_blank [])
+            ignore (Lang.apply ~t:Lang.unit_t on_blank [])
           end
       end
 end
 
 let log = Dtools.Log.make ["noblank"]
 
-class skip ~length ~threshold source =
+class skip ~kind ~length ~threshold ~track_sensitive source =
 object (self)
-  inherit operator [source]
+  inherit operator kind [source]
   inherit base ~length ~threshold as base
 
   method stype = source#stype
   method is_ready = source#is_ready
-  method abort_track = 
-    source#abort_track ; 
+  method abort_track =
+    source#abort_track ;
     base#abort_track
   method remaining = source#remaining
 
@@ -106,7 +109,9 @@ object (self)
   method private get_frame ab =
     let p0 = AFrame.position ab in
       source#get ab ;
-      if AFrame.is_partial ab || p0 > 0 then blank_len <- 0 else begin
+      if track_sensitive && (AFrame.is_partial ab || p0 > 0) then
+        blank_len <- 0
+      else begin
         self#check_blank ab p0 ;
         if blank_len > length then begin
           source#abort_track ;
@@ -115,14 +120,18 @@ object (self)
       end
 end
 
-class strip ~length ~threshold source =
+class strip ~kind ~length ~threshold ~track_sensitive source =
 object (self)
-  inherit active_operator source
+  inherit active_operator kind [source]
   inherit base ~length ~threshold as base
 
   val mutable stripping = false
 
-  val mutable ns = []
+  initializer
+    ns_kind <- "strip_blank" ;
+    let status _ = string_of_bool stripping in
+    self#register_command
+      "is_stripping" ~descr:"Check if the source is stripping." status
 
   method stype = Fallible
   method is_ready = not stripping && source#is_ready
@@ -138,7 +147,7 @@ object (self)
     let p0 = AFrame.position ab in
     let b0 = AFrame.breaks ab in
       source#get ab ;
-      if AFrame.is_partial ab || p0 > 0 then begin
+      if track_sensitive && (AFrame.is_partial ab || p0 > 0) then begin
         blank_len <- 0 ;
         stripping <- false
       end else begin
@@ -152,20 +161,14 @@ object (self)
   method private output =
     if stripping && AFrame.is_partial memo then self#get_frame memo
 
-  method output_get_ready = 
-    if ns = [] then
-      ns <- Server.register [self#id] "strip_blank" ;
-    let status _ = string_of_bool stripping in
-    Server.add
-      ~ns "is_stripping" ~descr:"Check if the source is stripping." status 
-
   method output_reset = ()
+  method output_get_ready = ()
   method is_active = true
 end
 
-class eat ~at_beginning ~length ~threshold source =
+class eat ~kind ~at_beginning ~length ~threshold ~track_sensitive source =
 object (self)
-  inherit operator [source] as super
+  inherit operator kind [source] as super
   inherit base ~length ~threshold as base
 
   val mutable stripping = false
@@ -180,19 +183,19 @@ object (self)
     stripping <- false ;
     beginning <- true
 
-  method private on_noise = 
+  method private on_noise =
     stripping <- false ;
     beginning <- false
 
   method private get_frame ab =
     let first = ref true in
-    let breaks = ref (AFrame.breaks ab) in
+    let breaks = AFrame.breaks ab in
       while !first || stripping do
-        if not !first then AFrame.set_breaks ab !breaks;
+        if not !first then AFrame.set_breaks ab breaks;
         first := false;
         let p0 = AFrame.position ab in
           source#get ab ;
-          if AFrame.is_partial ab then
+          if track_sensitive && AFrame.is_partial ab then
             (
               blank_len <- 0 ;
               stripping <- false ;
@@ -207,31 +210,36 @@ object (self)
       done
 end
 
+let kind = Lang.kind_type_of_kind_format ~fresh:1 Lang.any_fixed
 let proto =
   [ "threshold", Lang.float_t, Some (Lang.float (-40.)),
     Some "Power in decibels under which the stream is considered silent." ;
     "length", Lang.float_t, Some (Lang.float 20.),
     Some "Maximum silence length allowed, in seconds." ;
-    "", Lang.source_t, None, None ]
+    "track_sensitive", Lang.bool_t, Some (Lang.bool true),
+    Some "Reset blank counter at each track.";
+    "", Lang.source_t kind, None, None ]
 
 let extract p =
   let f v = List.assoc v p in
   let s = Lang.to_source (f "") in
   let length =
     let l = Lang.to_float  (f "length") in
-      Fmt.samples_of_seconds l
+      Frame.audio_of_seconds l
   in
   let threshold =
     let v = f "threshold" in
     let t = Lang.to_float v in
       if t>0. then
         raise (Lang.Invalid_value (v,"threshold should be negative")) ;
-      Sutils.lin_of_dB t
+      Audio.lin_of_dB t
   in
-    length,threshold,s
+  let ts = Lang.to_bool (f "track_sensitive") in
+  length,threshold,ts,s
 
 let () =
   Lang.add_operator "on_blank"
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.TrackProcessing
     ~descr:"Calls a given handler when detecting a blank."
     (("", Lang.fun_t [] Lang.unit_t, None,
@@ -240,33 +248,40 @@ let () =
       Some (Lang.val_cst_fun [] Lang.unit),
       Some "Handler called when noise is detected.")::
      proto)
-    (fun p _ ->
+    (fun p kind ->
        let on_blank = Lang.assoc "" 1 p in
        let on_noise = Lang.assoc "on_noise" 1 p in
        let p = List.remove_assoc "" p in
-       let length,threshold,s = extract p in
-         new on_blank ~length ~threshold ~on_blank ~on_noise s) ;
+       let length,threshold,track_sensitive,s = extract p in
+         new on_blank ~kind ~length ~threshold
+           ~track_sensitive ~on_blank ~on_noise s) ;
   Lang.add_operator "skip_blank"
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.TrackProcessing
     ~descr:"Skip track when detecting a blank."
     proto
-    (fun p _ ->
-       let length,threshold,s = extract p in
-         new skip ~length ~threshold s) ;
+    (fun p kind ->
+       let length,threshold,track_sensitive,s = extract p in
+         new skip ~kind ~length ~threshold ~track_sensitive s) ;
   Lang.add_operator "strip_blank"
+    (* TODO make it active again when it doesn't hurt type
+     *   inference as much, cf. LS-551.
+     * ~active:true *)
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.TrackProcessing
     ~descr:"Make the source unavailable when it is streaming blank."
     proto
-    (fun p _ ->
-       let length,threshold,s = extract p in
-         ((new strip ~length ~threshold s):>source));
+    (fun p kind ->
+       let length,threshold,track_sensitive,s = extract p in
+         ((new strip ~kind ~length ~threshold ~track_sensitive s):>source));
   Lang.add_operator "eat_blank"
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.TrackProcessing
     ~descr:"Eat blanks, i.e., drop the contents of the stream until \
             it is not blank anymore."
     (("at_beginning", Lang.bool_t, Some (Lang.bool false),
       Some "Only eat at the beginning of a track.")::proto)
-    (fun p _ ->
+    (fun p kind ->
        let at_beginning = Lang.to_bool (List.assoc "at_beginning" p) in
-       let length,threshold,s = extract p in
-         new eat ~at_beginning ~length ~threshold s)
+       let length,threshold,track_sensitive,s = extract p in
+         new eat ~kind ~at_beginning ~length ~threshold ~track_sensitive s)
