@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,9 +20,9 @@
 
  *****************************************************************************)
 
-class on_end ~delay f s =
+class on_end ~kind ~delay f s =
 object (self)
-  inherit Source.operator [s]
+  inherit Source.operator kind [s]
 
   val mutable executed = false
 
@@ -32,24 +32,23 @@ object (self)
   method is_ready = s#is_ready
   method remaining = s#remaining
   method abort_track = s#abort_track
+  method seek n = s#seek n
 
   method private get_frame ab =
     s#get ab ;
     let compare x y = - (compare x y) in
-    let l = 
-      List.sort compare (Frame.get_all_metadata ab) 
-    in
+    let l = List.sort compare (Frame.get_all_metadata ab) in
     if List.length l > 0 then
       latest_metadata <- Some (snd (List.hd l));
-    let rem = Fmt.seconds_of_ticks s#remaining in
+    let rem = Frame.seconds_of_master s#remaining in
     if rem <= delay && not executed then
     begin
-      let m = 
+      let m =
         match latest_metadata with
           | Some m -> m
           | None -> Hashtbl.create 0
       in
-      ignore(Lang.apply f ["",Lang.float rem;
+      ignore(Lang.apply ~t:Lang.unit_t f ["",Lang.float rem;
                            "",Lang.metadata m]) ;
       executed <- true
     end ;
@@ -58,25 +57,27 @@ object (self)
 end
 
 let () =
+  let kind = Lang.univ_t 1 in
   Lang.add_operator "on_end"
     [ "delay", Lang.float_t,
       Some (Lang.float 5.),
-      Some ("Execute handler when remaining time is less or \
-             equal to this value.") ;
+      Some "Execute handler when remaining time is less or \
+            equal to this value." ;
       "",
       Lang.fun_t
         [(false,"",Lang.float_t);
           false,"",Lang.list_t (Lang.product_t Lang.string_t Lang.string_t)]
         Lang.unit_t,
       None,
-      Some ("Function to execute. First argument is the remaining time, \
-             second is the latest metadata.") ;
-      "", Lang.source_t, None, None ]
+      Some "Function to execute. First argument is the remaining time, \
+            second is the latest metadata." ;
+      "", Lang.source_t kind, None, None ]
     ~category:Lang.TrackProcessing
-    ~descr:"Call a given handler when there is less than a given amount of time \
-            remaining before then end of track."
-    (fun p _ ->
-       let delay = Lang.to_float (List.assoc "delay" p) in 
+    ~descr:"Call a given handler when there is less than \
+            a given amount of time remaining before then end of track."
+    ~kind:(Lang.Unconstrained kind)
+    (fun p kind ->
+       let delay = Lang.to_float (List.assoc "delay" p) in
        let f = Lang.assoc "" 1 p in
        let s = Lang.to_source (Lang.assoc "" 2 p) in
-         new on_end ~delay f s)
+         new on_end ~kind ~delay f s)
