@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,9 +22,11 @@
 
 open Source
 
-class effect effect (source:source) =
+module Img = Image.RGBA32
+
+class effect ~kind effect (source:source) =
 object (self)
-  inherit operator [source] as super
+  inherit operator kind [source] as super
 
   method stype = source#stype
 
@@ -34,81 +36,87 @@ object (self)
   method abort_track = source#abort_track
 
   method private get_frame buf =
-    let offset = AFrame.position buf in
-      source#get buf ;
-      let b = VFrame.get_rgb buf in
-      let position = VFrame.position buf in
-        for c = 0 to Array.length b - 1 do
-          let bc = b.(c) in
-            for i = offset to position - 1 do
-              effect bc.(i)
+    match VFrame.get_content buf source with
+      | None -> ()
+      | Some (rgb,offset,length) ->
+          let rgb = rgb.(0) in
+            for i = offset to offset + length - 1 do
+              effect rgb.(i)
             done
-        done
 end
 
+let kind =
+  Lang.kind_type_of_kind_format ~fresh:1 (Lang.any_fixed_with ~video:1 ())
+
 let () =
   Lang.add_operator "video.greyscale"
-    [ "", Lang.source_t, None, None ]
+    [ "", Lang.source_t kind, None, None ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Convert video to greyscale."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let src = Lang.to_source (f "") in
-         new effect RGB.greyscale src)
+         new effect ~kind Img.Effect.greyscale src)
 
 let () =
   Lang.add_operator "video.sepia"
-    [ "", Lang.source_t, None, None ]
+    [ "", Lang.source_t kind, None, None ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Convert video to sepia."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let src = Lang.to_source (f "") in
-         new effect RGB.sepia src)
+         new effect ~kind Img.Effect.sepia src)
 
 let () =
   Lang.add_operator "video.invert"
-    [ "", Lang.source_t, None, None ]
+    [ "", Lang.source_t kind, None, None ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Invert video."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let src = Lang.to_source (f "") in
-         new effect RGB.invert src)
+         new effect ~kind Img.Effect.invert src)
 
 let () =
   Lang.add_operator "video.opacity"
     [
       "", Lang.float_t, None, Some "Coefficient to scale opacity with.";
-      "", Lang.source_t, None, None
+      "", Lang.source_t kind, None, None
     ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Scale opacity of video."
-    (fun p _ ->
+    (fun p kind ->
        let a = Lang.to_float (Lang.assoc "" 1 p) in
        let src = Lang.to_source (Lang.assoc "" 2 p) in
-         new effect (fun buf -> RGB.scale_opacity buf a) src)
+         new effect ~kind (fun buf -> Img.Effect.Alpha.scale buf a) src)
 
 let () =
   Lang.add_operator "video.opacity.blur"
     [
-      "", Lang.source_t, None, None
+      "", Lang.source_t kind, None, None
     ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Blur opacity of video."
-    (fun p _ ->
+    (fun p kind ->
        let src = Lang.to_source (Lang.assoc "" 1 p) in
-         new effect RGB.blur_alpha src)
+         new effect ~kind Img.Effect.Alpha.blur src)
 
 let () =
   Lang.add_operator "video.lomo"
-    [ "", Lang.source_t, None, None ]
+    [ "", Lang.source_t kind, None, None ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Emulate the \"Lomo effect\"."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let src = Lang.to_source (f "") in
-         new effect RGB.lomo src)
+         new effect ~kind Img.Effect.lomo src)
 
 let () =
   Lang.add_operator "video.transparent"
@@ -120,11 +128,12 @@ let () =
       "color", Lang.int_t, Some (Lang.int 0),
       Some "Color which should be transparent (in 0xRRGGBB format).";
 
-      "", Lang.source_t, None, None
+      "", Lang.source_t kind, None, None
     ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Set a color to be transparent."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let prec, color, src =
          Lang.to_float (f "precision"),
@@ -132,8 +141,8 @@ let () =
          Lang.to_source (f "")
        in
        let prec = int_of_float (prec *. 255.) in
-       let color = RGB.rgb_of_int color in
-         new effect (fun buf -> RGB.color_to_alpha buf color prec) src)
+       let color = Image.RGB8.Color.of_int color in
+         new effect ~kind (fun buf -> Img.Effect.Alpha.of_color buf color prec) src)
 
 let () =
   Lang.add_operator "video.fill"
@@ -141,18 +150,19 @@ let () =
       "color", Lang.int_t, Some (Lang.int 0),
       Some "Color to fill the image with.";
 
-      "", Lang.source_t, None, None
+      "", Lang.source_t kind, None, None
     ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Fill frame with a color."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let color, src =
          Lang.to_int (f "color"),
          Lang.to_source (f "")
        in
-       let r,g,b = RGB.rgb_of_int color in
-         new effect (fun buf -> RGB.fill buf (r, g, b, 0xff)) src)
+       let r,g,b = Image.RGB8.Color.of_int color in
+         new effect ~kind (fun buf -> Img.fill_all buf (r, g, b, 0xff)) src)
 
 let () =
   Lang.add_operator "video.scale"
@@ -164,11 +174,12 @@ let () =
       "coef_y", Lang.float_t, Some (Lang.float 1.), Some "y scaling";
       "offset_x", Lang.int_t, Some (Lang.int 1), Some "x offset";
       "offset_y", Lang.int_t, Some (Lang.int 1), Some "y offset";
-      "", Lang.source_t, None, None
+      "", Lang.source_t kind, None, None
     ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Scale and translate video."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let src = Lang.to_source (f "") in
        let c, cx, cy, ox, oy =
@@ -178,12 +189,13 @@ let () =
          Lang.to_int (f "offset_x"),
          Lang.to_int (f "offset_y")
        in
-         new effect (fun buf -> RGB.affine buf (c*.cx) (c*.cy) ox oy) src)
+         new effect ~kind
+           (fun buf -> Img.Effect.affine buf (c*.cx) (c*.cy) ox oy) src)
 
 let () =
   let effect a da buf =
     a := !a +. da;
-    RGB.rotate buf !a
+    Img.Effect.rotate buf !a
   in
   Lang.add_operator "video.rotate"
     [
@@ -193,14 +205,15 @@ let () =
       "speed", Lang.float_t, Some (Lang.float 3.1416),
       Some "Rotation speed in radians per sec.";
 
-      "", Lang.source_t, None, None
+      "", Lang.source_t kind, None, None
     ]
+    ~kind:(Lang.Unconstrained kind)
     ~category:Lang.VideoProcessing
     ~descr:"Rotate video."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let src = Lang.to_source (f "") in
        let angle = ref (Lang.to_float (f "angle")) in
        let speed = Lang.to_float (f "speed") in
-       let da = speed /. float (Fmt.video_frames_per_second ()) in
-         new effect (effect angle da) src)
+       let da = speed /. float (Lazy.force Frame.video_rate) in
+         new effect ~kind (effect angle da) src)
