@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,44 +20,114 @@
 
  *****************************************************************************)
 
-open Lastfm
-open Lastfm.Audioscrobbler
 open Duppy
 
+let log = Dtools.Log.make ["liqfm"]
+
+(* A custom implementation of HTTP 
+ * requests. *)
+module Liq_http =
+ struct 
+  type request = Get | Post of string
+
+  exception Http of string
+
+  let exc_of_exc = 
+    function
+      | Http s -> Http s
+      | e -> Http (Utils.error_message e)
+
+  (* This in unused for now.. *)
+  let default_timeout = ref 5.
+  let request ?timeout ?headers ?port 
+              ~host ~url ~request () =
+  try
+    let log = log#f 4 "%s" in
+    let timeout = 
+      match timeout with
+        | None -> !default_timeout
+        | Some t -> t
+    in
+    let request = 
+      match request with
+        | Get -> Http.Get
+        | Post s -> Http.Post s
+    in
+    let (x,code,y),_,data = 
+          Http.full_request ?headers ?port ~log
+                            ~timeout ~host ~url 
+                            ~request ()
+    in
+    if code <> 200 then 
+      raise (Http (Printf.sprintf "Http request failed: %s %i %s" x code y));
+    data
+  with
+    | e -> raise (exc_of_exc e)
+ end
+
+module Audioscrobbler = Lastfm_generic.Audioscrobbler_generic(Liq_http)
+module Radio          = Lastfm_generic.Radio_generic(Liq_http)
+
+let error_translator =
+  function
+    | Audioscrobbler.Error x ->
+       raise (Utils.Translation
+          (Printf.sprintf "Audioscrobbler error: %s" 
+             (Audioscrobbler.string_of_error x)))
+    | Radio.Error x ->
+       raise (Utils.Translation
+          (Printf.sprintf "Lastfm radio error: %s"
+             (Radio.string_of_error x)))
+    | _ -> ()
+
+let () = Utils.register_error_translator error_translator
+
+open Lastfm_generic
+open Audioscrobbler
+
 type source = User | Lastfm | Broadcast | Recommendation | Unknown
 
 type submission = NowPlaying | Played
 
+type task =
+  { task : Duppy.Async.t ;
+    submit_m : Mutex.t ;
+    submissions : (string * string * source * submission *
+          bool * (string, string) Hashtbl.t) Queue.t ;
+  }
+
 let log = Dtools.Log.make ["audioscrobbler"]
 
 exception Duration
 
-(* The list of waiting submitions *)
-let submissions = Queue.create ()
-(* A mutex to manage thread concurrency *)
-let submit_m = Mutex.create ()
-(* A reference to the task *)
-let task = ref None
+let conf_liqfm =
+  Dtools.Conf.void ~p:(Configure.conf#plug "audioscrobbler")
+	    "Audioscrobbler configuration."
 
 let client = { client = "lsp"; version = "0.1" }
 
-let init () =
+let init host =
+ (* The list of waiting submitions *)
+ let submissions = Queue.create () in
+ (* A mutex to manage thread concurrency *)
+ let submit_m = Mutex.create () in
  let reason = log#f 3 "Lastfm Submission failed: %s" in
  (* Define a new task *)
  let rec do_submit () =
    try
     (* This function checks that the submission is valid *)
-    let song songs (user,password,host,source,stype,length,m) =
+    let song songs (user,password,source,stype,length,m) =
       let login = { user = user ; password = password } in
       let f = fun x -> try Hashtbl.find m x with Not_found -> "" in
       let artist,track = f "artist",f "title" in
       let s = 
         match stype with
-          | Played -> ""
-          | NowPlaying -> " (nowplaying)"
+          | Played -> "submit"
+          | NowPlaying -> "nowplaying"
       in
-      log#f 4 "Submiting %s -- %s%s" 
-         artist track s;
+      let (h,p) = host in
+      log#f 4 "Submiting %s -- %s with mode: %s to %s:%i" 
+         artist track s h p;
       try
         let duration () =
           try
@@ -98,11 +168,11 @@ let init () =
             | _ -> None
         in
         let source = match source with
-                       | User -> Lastfm.Audioscrobbler.User
-                       | Lastfm -> Lastfm.Audioscrobbler.Lastfm
-                       | Broadcast -> Lastfm.Audioscrobbler.Broadcast
-                       | Recommendation -> Lastfm.Audioscrobbler.Recommendation
-                       | Unknown -> Lastfm.Audioscrobbler.Unknown
+                       | User -> Audioscrobbler.User
+                       | Lastfm -> Audioscrobbler.Lastfm
+                       | Broadcast -> Audioscrobbler.Broadcast
+                       | Recommendation -> Audioscrobbler.Recommendation
+                       | Unknown -> Audioscrobbler.Unknown
         in
         let song = { artist = artist ; track = track ;
                      time = Some time  ;
@@ -112,7 +182,7 @@ let init () =
                      musicbrainzid = None ; trackauth = trackauth }
         in
         check_song song Submit ;
-        (login,host,stype,song) :: songs
+        (login,stype,song) :: songs
       with
         | Duration ->
             log#f 4 "could not submit track %s -- %s, no duration available"
@@ -124,7 +194,7 @@ let init () =
             songs
         | e ->
             log#f 4 "could not submit track %s -- %s: unknown error %s"
-              artist track (Printexc.to_string e) ;
+              artist track (Utils.error_message e) ;
             songs
      in
      Mutex.lock submit_m;
@@ -132,46 +202,42 @@ let init () =
      Queue.clear submissions ;
      Mutex.unlock submit_m;
      let submit = Hashtbl.create 10 in
-     let filter (c,h,t,m) =
+     let filter (c,t,m) =
        try
-         let v = Hashtbl.find submit (c,h,t) in
-         Hashtbl.replace submit (c,h,t) (m::v)
+         let v = Hashtbl.find submit (c,t) in
+         Hashtbl.replace submit (c,t) (m::v)
        with
-         | Not_found -> Hashtbl.add submit (c,h,t) [m]
+         | Not_found -> Hashtbl.add submit (c,t) [m]
      in
      List.iter filter songs ;
-     let f (login,host,stype) songs =
+     let f (login,stype) songs =
        try
          match stype with
            | NowPlaying -> 
                List.iter (fun song -> 
-                    Lastfm.Audioscrobbler.do_np ?host client login song)
+                    Audioscrobbler.do_np ~host client login song)
                     songs
            | Played ->
-              ignore(Lastfm.Audioscrobbler.do_submit ?host client login songs)
+              ignore(Audioscrobbler.do_submit ~host client login songs)
        with
-         | Lastfm.Audioscrobbler.Error e ->
-             reason (Lastfm.Audioscrobbler.string_of_error e)
-         | Lastfm.Radio.Error e ->
-             reason (Lastfm.Radio.string_of_error e)
+         | Audioscrobbler.Error e ->
+             reason (Audioscrobbler.string_of_error e)
      in
      Hashtbl.iter f submit ;
      (-1.)
    with
-     | e -> reason (Printexc.to_string e); raise e
+     | e -> reason (Utils.error_message e); (-1.)
    in
-   (* Add and wake the task *)
-   let t =
+   let task = 
      Duppy.Async.add ~priority:Tutils.Blocking Tutils.scheduler do_submit
    in
-   task := Some t;
-   Duppy.Async.wake_up t
+   { task = task ;
+     submit_m = submit_m ;
+     submissions = submissions }
 
-let submit ?host (user,password) length source stype songs =
- let songs = List.map (fun x -> (user,password,host,source,stype,length,x)) songs in
- Mutex.lock submit_m;
- List.iter (fun x -> Queue.add x submissions) songs ;
- Mutex.unlock submit_m;
- match !task with
-   | None -> init ()
-   | Some t -> Duppy.Async.wake_up t
+let submit (user,password) task length source stype songs =
+ let songs = List.map (fun x -> (user,password,source,stype,length,x)) songs in
+ Mutex.lock task.submit_m;
+ List.iter (fun x -> Queue.add x task.submissions) songs ;
+ Mutex.unlock task.submit_m;
+ Duppy.Async.wake_up task.task
