@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,10 +20,19 @@
 
  *****************************************************************************)
 
+module Audioscrobbler : Lastfm_generic.Audioscrobbler_t
+module Radio          : Lastfm_generic.Radio_t
+
 type source = User | Lastfm | Broadcast | Recommendation | Unknown
 
 type submission = NowPlaying | Played
 
+type task
+
+(** [init (host,port)]: Create a new task. *)
+val init : string*int -> task
+
 (** Performs a sumbission to audioscrobbler *)
-val submit : ?host:(string*int) -> string*string -> bool -> 
+val submit : string*string ->
+             task -> bool ->
              source -> submission -> Frame.metadata list -> unit
