@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,14 +23,20 @@
 (** Multithreading utilities *)
 
 (** {1 Thread wrapper}
-  Give names to threads, and forbid them to raise an exception.
-  The main process is expected to run raise_everything after having launched
-  the needed threads. So the main process will sleep, until a thread
-  aborts. In that case it will log and output the error, and return. *)
+  Give names to threads, and forbid them to raise an exception;
+  if that happens, the thread dies anyway but it is logged and [main]
+  will notice it.
+  The main process is expected to run [main] after having launched
+  the needed threads: that function will sleep until a thread
+  raises an exception. *)
 val create : ('a -> unit) -> 'a -> string -> Thread.t
 val main : unit -> unit
 val shutdown : unit -> unit
 
+(** Special exception allowed for "clean" termination of Tutils threads.
+  * All other exceptions are reported as bugs. *)
+exception Exit
+
 (** Wait for the threads to terminate,
   * never return if some thread keeps running. *)
 val join_all : unit -> unit
@@ -62,3 +68,29 @@ val wait : Condition.t -> Mutex.t -> (unit -> bool) -> unit
 
 (** Make a function work in critical section, protected by a given lock. *)
 val mutexify : Mutex.t -> ('a -> 'b) -> ('a -> 'b)
+
+(** [finalize ~k f] calls [f] and returns it result,
+  * and always executes [k], even when [f] raises an exception. *)
+val finalize : k:(unit -> unit) -> (unit -> 'a) -> 'a
+
+(** Tests whether a mutex is locked, without blocking.
+  * We cannot check on Win32, where [true] is always returned:
+  * it always "seems" OK, we don't raise false alarms.
+  * This is meant to be used for assertions. *)
+val seems_locked : Mutex.t -> bool
+
+(** Thread-safe equivalent to Lazy.lazy_from_fun. *)
+val lazy_cell : (unit -> 'a) -> (unit -> 'a)
+
+(** Preemptive stoppable thread.
+  *
+  * The thread function receives a [should_stop,has_stop] pair on startup.
+  * It should regularly poll the [should_stop] and stop when asked to.
+  * Before stopping it should call [has_stopped].
+  *
+  * The function returns a [kill,wait] pair. The first function should be
+  * called to request that the thread stops, and the second to wait
+  * that it has effectively stopped. *)
+val stoppable_thread :
+     (((unit -> bool) * (unit -> unit)) -> unit) -> string ->
+     (unit -> unit) * (unit -> unit)
