@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2011 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,9 +22,9 @@
 
 open Source
 
-class midimeter source =
+class midimeter ~kind source =
 object
-  inherit operator [source] as super
+  inherit operator kind [source] as super
 
   method stype = source#stype
   method is_ready = source#is_ready
@@ -32,33 +32,37 @@ object
   method abort_track = source#abort_track
 
   method get_frame buf =
+    let offset = MFrame.position buf in
     source#get buf;
-    let m = MFrame.tracks buf in
+    let m = MFrame.content buf offset in
+    (* Printf.printf "len: %d\n%!" (List.length (MIDI.data m.(0))); *)
       for c = 0 to Array.length m - 1 do
         List.iter
-          (fun (t, e) ->
+          (fun (_,e) ->
              let s =
                match e with
-                 | Midi.Note_on (n, v) ->
+                 | MIDI.Note_on (n, v) ->
                      Printf.sprintf "Note %d on at %.02f" n v
-                 | Midi.Note_off (n, v) ->
+                 | MIDI.Note_off (n, v) ->
                      Printf.sprintf "Note %d off at %.02f" n v
-                 | Midi.Control_change (c, v) ->
+                 | MIDI.Control_change (c, v) ->
                      Printf.sprintf "Control %x at %d" c v
                  | _ -> "???"
              in
                Printf.printf "%d: %s.\n%!" c s
-          ) !(m.(c))
+          ) (MIDI.data m.(c))
       done
 end
 
 let () =
+  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.any_fixed in
   Lang.add_operator "midimeter"
-    [ "", Lang.source_t, None, None ]
+    [ "", Lang.source_t k, None, None ]
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.Visualization
     ~flags:[Lang.Hidden; Lang.Experimental]
     ~descr:"Display midi events."
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
        let src = Lang.to_source (f "") in
-         ((new midimeter src):>Source.source))
+         ((new midimeter ~kind src):>Source.source))
